/********************************************************************************/
/* Copyright (c) 2007, SAS Institute Inc., Cary, NC, USA.  All rights reserved. */
/* NAME:        'UPDATESUPPRESSEDITRECS'                                        */
/*                                                                              */
/* PRODUCT:     SAS Fair Banking                                                */
/* PURPOSE:     updates Quality Suppression tables with records from            */
/*              the offline error correction spreadsheet                        */
/*                                                                              */
/* USAGE:        %updateSuppressEditRecs (                                      */
/*                            HMDA_or_CRA = &hmda_or_cra                        */
/*                          , tablenm     = &tgttbl                             */
/*                       ) ;                                                    */
/*                                                                              */
/* PARAMETERS:  I|O|U - comments                                                */
/* DOCUMENT:                                                                    */
/* TEST PROGRAM:                                                                */
/* NOTES:       called by %APPLY_EDITS_TO_REPOSITORY,                           */
/*                        %CRA_CORR_UPDT, %HMDA_CORR_UPDT                       */
/*                                                                              */
/* END                                                                          */
/********************************************************************************/
%macro updateSuppressEditRecs(HMDA_or_CRA=,
                              tablenm=,
                              suprmast=) /
    des='updates quality supression tables';

   %put NOTE: {START: &SYSMACRONAME} ;

    %* create macro var for each suppressable quality edit;

    %let nQEdits=0;


        proc sql ;
             /* find out govt and custom edits for the submission year */
         create table year_edit as
             (select distinct edit_id
          from l_sys.l_edit (where=(upcase(HMDA_or_CRA) = "%substr(%upcase(&HMDA_or_CRA),1,1)"
                                    and edit_type='Q'
                                    and &sub_year between start_year and end_year) )
          union select distinct edit_id
          from l_sys.l_custom_edit (where=(upcase(HMDA_or_CRA) = "%substr(%upcase(&HMDA_or_CRA),1,1)"
                                           and edit_type='Q'
                                           and &sub_year between start_year and end_year))
          );

                 /* obtain the edits that have validation reasons */
             create table suppressable_qedits_reasons as
             select distinct edit_id, reason_code, reason_desc
                 from l_sys.l_validation_reason
         where upcase(HMDA_or_CRA) = "%substr(%upcase(&HMDA_or_CRA),1,1)" and
                       edit_id in (select distinct edit_id from year_edit)
         ;
         create table suppressable_qedits as
                 select distinct edit_id
                 from suppressable_qedits_reasons
         ;
    quit;

    data _null_ ;
       set suppressable_qedits end = lr ;
       cnt + 1 ;
       ccnt  =  left (put (cnt, 4.)) ;
       call symput ('qedit' || ccnt, trim(left(edit_id))) ;
       if lr then call symput ('nQEdits', ccnt) ;
    run ;

  /* create a format table that maps the edit code to a reason key */
   data makefmt;
     set suppressable_qedits_reasons ;
     rename reason_code = start;
     fmtname = "%substr(%upcase(&HMDA_OR_CRA),1,1)"||trim(left(edit_ID))||'f';
     type = 'C';
     label=reason_desc;
     run;

     proc format cntlin=makefmt;
     run;

      data tmpAppend2SupprEdit (Keep= &HMDA_or_CRA._Record_Key
                                       HMDA_or_CRA edit_id reason_code);
          set &tablenm;
          if 0 then set &suprmast(obs=0);

          HMDA_or_CRA="%upcase(&HMDA_or_CRA)";


         %do i=1 %to &nQedits;

           edit_id="&&qedit&i";

           fmtname ='$'||substr( "%upcase(&HMDA_OR_CRA)",1,1)||trim(left(edit_ID))||'f';
                   /* 23dec05 special code to force Q027 when the employee loan and non-income qualifying loan indicator is yes */


           if edit_id ne '' and &&qedit&i ne ''  then
           do;

               /* 22dec05 check if the reason should be removed because edit was not triggered*/
                 sys_error_editkeys=''; /* initialize sys_error_editkeys to blank before testing the qedit */
                           if edit_id ne 'ALLQ' then /* do not check ALLQ */
                           do;
                              if edit_id ne 'Q027' then
                                  do;
                     %inc "&jobpath/%substr(&hmda_or_cra,1,1)_&&qedit&i...sas"; /* apply edit check */
                                  end;
                  else
                                   do;   
                         /* Q027 edit check should exclude the special indicators (employee_loan and non-incom qualifying loan) logic */
                                  if TRIM(Action_taken_type) IN('1','2','3','4','5','7','8') AND PROPERTY_TYPE IN ('1','2') and
                                         TRIM(Applicant_Income_in_k_c) EQ 'NA' and "&EXCLUDE_EMP" = "ON" then
                              sys_error_editkeys = 'Q027';
                                        end;
                           end;
                           else sys_error_editkeys='ALLQ';
               if sys_error_editkeys ne '' then
                           do;

                      if (putc(&&qedit&i,fmtname) ne &&qedit&i) or edit_id='ALLQ' /* 08jan08 S0338561 */then
                      do;
                         reason_code=&&qedit&i;
                         output tmpAppend2SupprEdit;
                      end;
                      else
                      do;
                         put '{ETLWARNING} Quality Suppression was not allowed because of invalid reason code.';
                     put '{ETLWARNING} ' &hmda_or_cra._record_key= edit_id= &&qedit&i=;
                      end;
                           end;
                           else
                            do;
                         put '{ETLWARNING} Quality Suppression was removed because the edit was not triggered.';
                         put '{ETLWARNING} ' &hmda_or_cra._record_key= edit_id= &&qedit&i=;
                      end;

           end;
         %end;

     run;

         %if %upcase(&hmda_or_cra)=HMDA %then
         %do;
             data tmpAppend2SupprEditQ027;
             set &tablenm;
                         /* 08jan06 changed where to if */
                     if Q027='' and
                               TRIM(Action_taken_type) IN('1','2','3','4','5','7','8') AND PROPERTY_TYPE IN ('1','2') and
                                   TRIM(Applicant_Income_in_k_c) EQ 'NA' and "&EXCLUDE_EMP" = "ON" and
                           ( upcase(non_income_qualifying_loan_ind)='Y' or upcase(employee_loan_indicator)='Y')

                     ;
                         hmda_or_cra='H';
                         keep hmda_or_cra hmda_record_key edit_id reason_code;
                         edit_id='Q027 ';
                     if   upcase(non_income_qualifying_loan_ind)='Y'  then reason_code='R011';
                     else if upcase(employee_loan_indicator)='Y'  then reason_code='R012';
                run;

                proc append base=tmpAppend2SupprEdit data=tmpAppend2SupprEditQ027 force;
                run;


         %end;


     proc sort data=tmpAppend2SupprEdit
               dupout=tmpAppend2SupprEdit_d nodupkey noequals;

          by &HMDA_or_CRA._Record_Key HMDA_or_CRA Edit_ID reason_code ;
     run;

  %put NOTE: {END: &SYSMACRONAME};
%mend;
